/*  RGB LED TABLE LAMP

    A table lamp made with RGB LEDs
    
    Hardware used: Atmega328P @ 16 MHz with external crystal + TLC5940
    
    Changelog:
    2012/06/08: v. 0.1.0 - First stable release
    2012/05/01: v. 0.0.1 - Very early release
    
    License:
    project released under the terms of the Creative Commons Share Alike-
    -No Commercial-Attribution 3.0 Ita licence
    
    Project by: Leonardo Miliani - www.leonardomiliani.com
*/

#include "CapSense.h" //to use the capacitative sensor
#include "Tlc5940.h" //to pilot the TLC5940

CapSense capSensor=CapSense(4,7); //cap sensor on pins 4 & 7
const byte whiteLeds=5;

//global variables
byte mult;
byte scheme=0;
byte chColor=0;
int pwm[3]= {0,0,0};
char sign[3]={1,1,1}; 
int sTimer=128;
unsigned long timer; 

void setup() {
    //initialize the PRNG
    randomSeed(analogRead(A0));

    //initialize the Tlc5940
    Tlc.init(0);
    
    //initialze the cap sensor
    capSensor.set_CS_AutocaL_Millis(0xFFFFFFFF);
    
    //initial scheme
    mult=10;
    timer=millis();
    scheme=1;
}


void loop(){
    byte j, temp;
    byte i;
    char a;

    //check if it's time to update the LEDs
    if (millis() > timer) {
        timer = millis()+(sTimer*mult);
        switch (scheme) {
            case 0: //tueverything is OFF
                Tlc.setAll(0);
                Tlc.update();
                analogWrite(whiteLeds, 0);
                break;

            case 1: //steady red
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=0; i<16; i+=3) {
                    Tlc.set(i, 4095);
                }
                Tlc.update();
                break;
                
            case 2: //steady green
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=1; i<16; i+=3) {
                    Tlc.set(i, 4095);
                }
                Tlc.update();
                break;
                
            case 3: //steady blue
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=2; i<16; i+=3) {
                    Tlc.set(i, 4095);
                    }
                Tlc.update();
                break;
            
            case 4: //steady yellow
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=0; i<16; i+=3) {
                    Tlc.set(i, 4095);
                    Tlc.set(i+1, 4095);
                }
                Tlc.update();
                break;
                
            case 5: //steady purple
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=0; i<16; i+=3) {
                    Tlc.set(i, 4095);
                    Tlc.set(i+2, 4095);
                }
                Tlc.update();
                break;
                
            case 6: //steady cyan
                Tlc.setAll(0);
                analogWrite(whiteLeds, 0);
                for (i=0; i<16; i+=3) {
                    Tlc.set(i+1, 4095);
                    Tlc.set(i+2, 4095);
                }
                Tlc.update();
                break;
                
            case 7: //color fading
                analogWrite(whiteLeds, 0);
                switch (chColor) {
                    case 0:
                        writeAnalog(pwm[0], 0, 0);
                        break;
                    case 1:
                        writeAnalog(0, pwm[0], 0);
                        break;
                    case 2:
                        writeAnalog(0, 0, pwm[0]);
                        break;
                    case 3:
                        writeAnalog(pwm[0], 0, pwm[0]);
                        break;
                    case 4:
                        writeAnalog(pwm[0], pwm[0], 0);
                        break;
                    case 5:
                        writeAnalog(0, pwm[0], pwm[0]);
                        break;
                    case 6:
                        writeAnalog(pwm[0], pwm[0], pwm[0]);
                        break;
                }
                a=8*sign[0];
                if ((pwm[0]+a)>4095) {
                    sign[0]*=-1;
                    a*=-1;
                } 
                if ((pwm[0]+a)<0) {
                    sign[0]*=-1;
                    a*=-1;
                    chColor++;
                    if (chColor>6) {
                        chColor=0;
                    }
                }
                pwm[0]+=a;
                break;
                
            case 8:
                //switch between random colors
                analogWrite(whiteLeds, 0);
                writeAnalog(random(0, 4096), random(0, 4096), random(0, 4096));
                break;
               
            case 9:
                //random fadings
                analogWrite(whiteLeds, 0);
                writeAnalog(pwm[0], pwm[1], pwm[2]);
                for (i=0; i<3; i++) {
                    a=8*sign[i];
                    if (((pwm[i]+a)>4095) || ((pwm[i]+a)<0) ) {
                        sign[i]*=-1;
                        a*=-1;
                    }
                    pwm[i]+=a;
                }
                break;
                
            case 10: //a random color each time
                analogWrite(whiteLeds, 0);
                j=random(0,16);
                i=random(0, 4096);
                Tlc.set(j, i);
                Tlc.update();
                delay(100);
                break;        
                                
            case 11: //only white LEDs
                analogWrite(whiteLeds, 255);
                Tlc.setAll(0);
                Tlc.update();
                break;
            
            case 12: //all the LEDs are ON
                analogWrite(whiteLeds, 255);
                Tlc.setAll(4095);
                Tlc.update();
                break;
        }
    }

    pressure();
}


void pressure() {
    unsigned long reading;
    
    reading=capSensor.capSense(10);
    //Serial.println(lettura);
    if (reading>=60) {
        while (capSensor.capSense(10)>60) {} //finto loop per attendere l'allontanamento del dito
        scheme+=1;
        if (scheme>12) {
            scheme=0;
        }
        switch (scheme) {
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
                mult=10;
                break;
            case 7:
                mult=1;
                break;
            case 8:
                mult=5;
                break;
            case 9:
                mult=10;
                break;
            case 10:
                mult=5;
                break;
            case 11:
            case 12:
                mult=1;
                break;
        }
        delay(250);
    }
}


void writeAnalog(int state1, int state2, int state3) {
    for (byte i=0; i<16; i+=3) {
        Tlc.set(i, state1);
        Tlc.set(i+1, state2);
        Tlc.set(i+2, state3);
    }
    Tlc.update();
}
